# pipeline
from sklearn.ensemble import RandomForestClassifier
from sklearn.pipeline import Pipeline

# feature scaling
from sklearn.preprocessing import MinMaxScaler

from credit_scoring_model.config.core import config

# set up the pipeline
credit_scoring_pipe = Pipeline(
    [
        # scale
        ("scaler", MinMaxScaler()),
        (
            "RandomForestClassifier",
            RandomForestClassifier(
                criterion=config.model_config.criterion,
                min_samples_leaf=config.model_config.min_samples_leaf,
                min_samples_split=config.model_config.min_samples_split,
                n_estimators=config.model_config.n_estimators,
            ),
        ),
    ]
)



import logging
from pathlib import Path

import numpy as np
import pandas as pd
from config.core import LOG_DIR, DATASET_DIR, config
from imblearn.over_sampling import RandomOverSampler
from pipeline import credit_scoring_pipe
from processing.data_manager import load_train_dataset, save_pipeline
from sklearn.metrics import mean_squared_error
from sklearn.model_selection import train_test_split
from sklearn.preprocessing import LabelEncoder

from credit_scoring_model import __version__ as _version


def run_training() -> None:
    """Train the model."""
    # Update logs
    log_path = Path(f"{LOG_DIR}/log_{_version}.log")
    if Path.exists(log_path):
        log_path.unlink()
    logging.basicConfig(filename=log_path, level=logging.DEBUG)

    # read training data
    data = load_train_dataset(file_name=config.app_config.data_file_train)

    print(data)
    data["age"] = data["age"].astype("int")
    data["job"] = data["job"].astype("object")
    data["marital"] = data["marital"].astype("object")
    data["education"] = data["education"].astype("object")
    data["default"] = data["default"].astype("object")
    data["balance"] = data["balance"].astype("int")
    data["housing"] = data["housing"].astype("object")
    data["loan"] = data["loan"].astype("object")
    data["contact"] = data["contact"].astype("object")
    data["day"] = data["day"].astype("int")
    data["month"] = data["month"].astype("object")
    data["duration"] = data["duration"].astype("int")
    data["campaign"] = data["campaign"].astype("int")
    data["pdays"] = data["pdays"].astype("int")
    data["previous"] = data["previous"].astype("int")
    data["poutcome"] = data["poutcome"].astype("object")

    lst = ["job", "marital", "education", "default", "housing", "housing", "loan", "contact", "month", "poutcome", "y"]

    le = LabelEncoder()

    for i in lst:
        data[i] = le.fit_transform(data[i])

    x = data.iloc[:, :-1]
    y = data.iloc[:, -1]

    sm = RandomOverSampler()
    x, y = sm.fit_resample(x, y)

    # divide train and test
    X_train, X_test, y_train, y_test = train_test_split(
        x,  # predictors
        y,
        test_size=config.model_config.test_size,
        random_state=config.model_config.random_state,
    )

    # fit model
    credit_scoring_pipe.fit(X_train, y_train)

    # make predictions for train set
    pred = credit_scoring_pipe.predict(X_train)

    mse_train = mean_squared_error(y_train, pred)
    rmse_train = np.sqrt(mse_train)

    print(f"train RMSE: {rmse_train}")
    print()

    logging.info(f"train RMSE: {rmse_train}")

    # make predictions for test set
    pred = credit_scoring_pipe.predict(X_test)

    # determine test accuracy and roc-auc
    mse_test = mean_squared_error(y_test, pred)
    rmse_test = np.sqrt(mse_test)

    print(f"test RMSE: {rmse_test}")
    print()

    logging.info(f"test RMSE: {rmse_test}")

    # persist trained model
    save_pipeline(pipeline_to_persist=credit_scoring_pipe)

    X_train.to_csv(f'{DATASET_DIR}/xtrain.csv', index=False)
    X_test.to_csv(f'{DATASET_DIR}/xtest.csv', index=False)

    y_train.to_csv(f'{DATASET_DIR}/ytrain.csv', index=False)
    y_test.to_csv(f'{DATASET_DIR}/ytest.csv', index=False)


if __name__ == "__main__":
    run_training()



import re
from typing import List, Optional, Tuple

import numpy as np
import pandas as pd
from pydantic import BaseModel, Field, ValidationError

from credit_scoring_model.config.core import config


def validate_inputs(*, input_data: pd.DataFrame) -> Tuple[pd.DataFrame, Optional[dict]]:
    input_data["age"] = input_data["age"].astype("int")
    input_data["job"] = input_data["job"].astype("int")
    input_data["marital"] = input_data["marital"].astype("int")
    input_data["education"] = input_data["education"].astype("int")
    input_data["default"] = input_data["default"].astype("int")
    input_data["balance"] = input_data["balance"].astype("int")
    input_data["housing"] = input_data["housing"].astype("int")
    input_data["loan"] = input_data["loan"].astype("int")
    input_data["contact"] = input_data["contact"].astype("int")
    input_data["day"] = input_data["day"].astype("int")
    input_data["month"] = input_data["month"].astype("int")
    input_data["duration"] = input_data["duration"].astype("int")
    input_data["campaign"] = input_data["campaign"].astype("int")
    input_data["pdays"] = input_data["pdays"].astype("int")
    input_data["previous"] = input_data["previous"].astype("int")
    input_data["poutcome"] = input_data["poutcome"].astype("int")
    relevant_data = input_data[config.model_config.features].copy()

    errors = None

    try:
        # replace numpy nans so that pydantic can validate
        MultipleCreditScoringInputs(inputs=relevant_data.replace({np.nan: None}).to_dict(orient="records"))
    except ValidationError as error:
        errors = error.json()

    return relevant_data, errors


class CreditScoringInputSchema(BaseModel):
    age: Optional[np.float64] = Field(None, alias="age")
    job: Optional[np.float64] = Field(None, alias="job")
    marital: Optional[np.float64] = Field(None, alias="marital")
    education: Optional[np.float64] = Field(None, alias="education")
    default: Optional[np.float64] = Field(None, alias="default")
    balance: Optional[np.float64] = Field(None, alias="balance")
    housing: Optional[np.float64] = Field(None, alias="housing")
    loan: Optional[np.float64] = Field(None, alias="loan")
    contact: Optional[np.float64] = Field(None, alias="contact")
    day: Optional[np.float64] = Field(None, alias="day")
    month: Optional[np.float64] = Field(None, alias="month")
    duration: Optional[np.float64] = Field(None, alias="duration")
    campaign: Optional[np.float64] = Field(None, alias="campaign")
    pdays: Optional[np.float64] = Field(None, alias="pdays")
    previous: Optional[np.float64] = Field(None, alias="previous")
    poutcome: Optional[np.float64] = Field(None, alias="poutcome")


class MultipleCreditScoringInputs(BaseModel):
    inputs: List[CreditScoringInputSchema]



import typing as t

import pandas as pd

from credit_scoring_model import __version__ as _version
from credit_scoring_model.config.core import config
from credit_scoring_model.processing.data_manager import load_pipeline
from credit_scoring_model.processing.validation import validate_inputs
from sklearn.preprocessing import MinMaxScaler

pipeline_file_name = f"{config.app_config.pipeline_save_file}{_version}.pkl"
_loan_approval_pipe = load_pipeline(file_name=pipeline_file_name)


def make_prediction(
    *,
    input_data: t.Union[pd.DataFrame, dict],
) -> dict:
    """Make a prediction using a saved model pipeline."""

    scaler = MinMaxScaler()

    scaler.fit(input_data)

    input_data = pd.DataFrame(
        scaler.transform(input_data),
        columns=input_data.columns
    )

    data = pd.DataFrame(input_data)

    validated_data, errors = validate_inputs(input_data=data)

    results: t.Dict[str, t.Any] = {"preds": None, "version": _version, "errors": errors}

    preds = _loan_approval_pipe.predict(X=validated_data[config.model_config.features])
    probs = _loan_approval_pipe.predict_proba(X=validated_data[config.model_config.features])[:, 1]

    # Fill the results dict
    results["preds"] = [pred for pred in preds]
    results["probs"] = [prob for prob in probs]

    return results



import numpy as np

from credit_scoring_model.config.core import config


def test_temporal_variable_transformer(sample_input_data):
    assert sample_input_data["balance"].iat[0] == 49
    assert sample_input_data["housing"].iat[3] == 1

    subject = sample_input_data

    # Then
    # test 1
    test_subject_1 = subject["balance"].iat[0]
    assert isinstance(test_subject_1, np.int64)
    assert test_subject_1 == 49

    # test 2
    test_subject_2 = subject["housing"].iat[3]
    assert isinstance(test_subject_2, np.int64)
    assert test_subject_2 == 1



import numpy as np
import pandas as pd

from credit_scoring_model.predict import make_prediction
from sklearn.preprocessing import MinMaxScaler

def test_make_prediction(sample_input_data):
    # Given
    expected_shape = (800, 16)

    scaler = MinMaxScaler()

    scaler.fit(sample_input_data)

    sample_input_data = pd.DataFrame(
        scaler.transform(sample_input_data),
        columns=sample_input_data.columns
    )

    # When
    result = make_prediction(input_data=sample_input_data)
    # Then
    preds = result.get("preds")
    probs = result.get("probs")

    assert isinstance(preds, list)
    assert isinstance(preds[0], np.int32)
    assert isinstance(probs[0], np.float64)
    assert result.get("errors") is None
    assert len(preds) == len(probs) == expected_shape[0]



import typing as t
from pathlib import Path

import joblib
import pandas as pd
from sklearn.pipeline import Pipeline

from credit_scoring_model import __version__ as _version
from credit_scoring_model.config.core import DATASET_DIR, TRAINED_MODEL_DIR, config


def load_train_dataset(*, file_name: str) -> pd.DataFrame:
    dataframe = pd.read_csv(Path(f"{DATASET_DIR}/{file_name}"), sep=";")
    return dataframe


def load_test_dataset(*, file_name: str) -> pd.DataFrame:
    dataframe = pd.read_csv(Path(f"{DATASET_DIR}/{file_name}"), sep=",")
    return dataframe


def save_pipeline(*, pipeline_to_persist: Pipeline) -> None:
    """Persist the pipeline.
    Saves the versioned model, and overwrites any previous
    saved models. This ensures that when the package is
    published, there is only one trained model that can be
    called, and we know exactly how it was built.
    """

    # Prepare versioned save file name
    save_file_name = f"{config.app_config.pipeline_save_file}{_version}.pkl"
    save_path = TRAINED_MODEL_DIR / save_file_name

    remove_old_pipelines(files_to_keep=[save_file_name])
    joblib.dump(pipeline_to_persist, save_path)


def load_pipeline(*, file_name: str) -> Pipeline:
    """Load a persisted pipeline."""

    file_path = TRAINED_MODEL_DIR / file_name
    trained_model = joblib.load(filename=file_path)
    return trained_model


def remove_old_pipelines(*, files_to_keep: t.List[str]) -> None:
    """
    Remove old model pipelines.
    This is to ensure there is a simple one-to-one
    mapping between the package version and the model
    version to be imported and used by other applications.
    """
    do_not_delete = files_to_keep + ["__init__.py"]
    for model_file in TRAINED_MODEL_DIR.iterdir():
        if model_file.name not in do_not_delete:
            model_file.unlink()
